--[[--

游戏主文件

]]

function __G__TRACKBACK__(msg)
    local errorMsg = ""
    local pid = ""

    if UserManager and UserManager.getPID then 
        pid = UserManager:getPID()
        if pid == -1 then pid = "" end 
    end 

    errorMsg = errorMsg .. "LUA ERROR: (" .. pid .. ")" .. tostring(msg) .. "\n"
    errorMsg = errorMsg .. debug.traceback()

    --request  
    if device and device.platform == "ios" and OPEN_ERROR_LOG and network and network.createHTTPRequest then 
        local req = network.createHTTPRequest(
            function() end, 
            "http://"..BUG_REPORT_IP..":"..BUG_REPORT_PORT.."/clientMsg/collect", 
            "POST"
        )
        req:addPOSTValue("msg", errorMsg)
        req:start()
    end 

    print("----------------------------------------")
    print(errorMsg)
    print(debug.traceback())
    print("----------------------------------------")
end

--[[--

main函数，游戏入口

]]

local function main()
    --防止内存泄露
    collectgarbage("setpause", 100)
    collectgarbage("setstepmul", 5000)
    
    --初始化脚本搜索路径    
    device = device or {}
    device.writablePath = CCFileUtils:sharedFileUtils():getWritablePath()
    CCFileUtils:sharedFileUtils():addSearchPath(device.writablePath.."Resources/wol/scripts/")
    CCFileUtils:sharedFileUtils():addSearchPath(device.writablePath.."Resources/wol/res/ccs/")
    CCFileUtils:sharedFileUtils():addSearchPath(device.writablePath.."Resources/wol/res/")
    if _LUAGIT_VERSION_ == 64 then 
        CCFileUtils:sharedFileUtils():addSearchPath("scripts64/")
    else 
        CCFileUtils:sharedFileUtils():addSearchPath("scripts/")
    end 
    CCFileUtils:sharedFileUtils():addSearchPath("res/ccs/")
    CCFileUtils:sharedFileUtils():addSearchPath("res/")

    --需要提前处理预加载
    require "lfs"

    local files = {}
    local scriptsPath = device.writablePath.."Resources/wol/scripts"

    --必须优先判断该文件夹存在
    if CCFileUtils:sharedFileUtils():isFileExist(scriptsPath) then 

        local function removePreloadFromPath(modelPath)
            if not modelPath or string.len(modelPath) == 0 then
                return
            end

            local modname = string.gsub(modelPath, "%.lua", "")
            modname = string.gsub(modname, "/", "%.")

            if modname and package.preload[modname] then 
                package.preload[modname] = nil
            end 
        end 

        local function _addFile(path, parentName)
            local iter, dir_obj = lfs.dir(path)

            while true do
                local dir = iter(dir_obj)
                if dir == nil then break end
                if dir ~= "." and dir ~= ".." and dir ~= ".DS_Store" then
                    local curDir = path.."/"..dir
                    local mode = lfs.attributes(curDir, "mode")

                    if mode == "directory" then
                        if parentName == "" then 
                            parentName = dir 
                        else 
                            parentName = parentName.."/"..dir
                        end 
                        _addFile(curDir, parentName)
                    elseif mode == "file" then
                        local scriptpath = ""
                        if parentName == "" then 
                            scriptpath = dir 
                        else 
                            scriptpath = parentName.."/"..dir
                        end 
                        removePreloadFromPath(scriptpath)
                    end
                end
            end
        end 

        _addFile(scriptsPath, "")

    end 

    --加载lua文件
    require "lib.Import"

    import "lib.core.S"
    import "lib.core.IO"
    import "lib.core.App"
    import "lib.core.AppEvent" 
    import "lib.core.AppNet"
    import "lib.core.ProtoBuff"
    import "lib.core.Element"
    import "lib.core.Net"
    import "lib.core.Packet"
    import "lib.core.Quad"
    import "lib.core.Box"
    import "lib.core.SocketNet"
    import "lib.core.Stage"
    import "lib.core.Main"

    --优先加载必要的库，更新后需清理重新加载
    require "dependencies"
    require "config"
    require "framework.init"
    require "framework.shortcodes"
    require "framework.cc.init" 
    require "framework.cc.ui.init"

    --for anaylitics
    require "dp.Anaylitics"
    require "common.Utils"
    require "lib.lang.lang"
    require "lib.lang.lang2"

    --加载闪屏
    local loadScene = require "loader.LoadScene"
    CCDirector:sharedDirector():runWithScene(loadScene)
    
end

xpcall(main, __G__TRACKBACK__)